import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Comparison', async () => {
    let parser: any;

    const values = `{
        "data_points": {
            "banana": {"score": "21.5" }
        }
    }`;

    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('Equals.true statements', async () => {
        it('Should return true when expression true == true', async () => {
            const expression = "true == true";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression false == false', async () => {
            const expression = "false == false";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression is size(data_points.banana.score) > 3 for custom type string', async () => {
            const expression = "size(data_points.banana.score) > 3";
            const expected = {bool_value: true};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 == 1', async () => {
            const expression = "1 == 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression dyn(1) == 1u', async () => {
            const expression = "dyn(1) == 1u";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression dyn(1) == 1.0', async () => {
            const expression = "dyn(1) == 1.0";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression banana == banana', async () => {
            const expression = '"banana" == "banana"';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression banana == r"banana"', async () => {
            const expression = '"banana" == r"banana"';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 == 1.0', async () => {
            const expression = "1 == 1.0";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1u == 1.0', async () => {
            const expression = "1u == 1.0";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 == 1u', async () => {
            const expression = "1 == 1u";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression dyn(1.0) == 1', async () => {
            const expression = "dyn(1.0) == 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1.0 == 1e+0', async () => {
            const expression = "1.0 == 1e+0";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 == 1.0e+0', async () => {
            const expression = "1 == 1.0e+0";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 == 1.0e+0', async () => {
            const expression = "1 == 1.0e+0";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression \'ίσος\' == \'ίσος\'', async () => {
            const expression = "'ίσος' == 'ίσος'";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression [1] == [1]', async () => {
            const expression = "[1] == [1]";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression null == null', async () => {
            const expression = "null == null";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression [] == []', async () => {
            const expression = "[] == []";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression [null] == [null]', async () => {
            const expression = "[null] == [null]";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression [1, 2, 3] == [1, 2, 3]', async () => {
            const expression = "[1, 2, 3] == [1, 2, 3]";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression [1.0, 2.0, 3] == [1u, 2, 3u]', async () => {
            const expression = "[1.0, 2.0, 3] == [1u, 2, 3u]";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression {\'k\':1.0} == {\'k\':1e+0}', async () => {
            const expression = "{'k':1.0} == {'k':1e+0}";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression {"1": 1.0, "2": 3u} == {"1": 1, "2": 3.0}', async () => {
            const expression = "{\"1\": 1.0, \"2\": 3u} == {\"1\": 1, \"2\": 3.0}";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression {\'k\':1.0} == {\'k\':1e+0}', async () => {
            const expression = "{'k':1.0} == {'k':1e+0}";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression {\'k1\':\'v1\',\'k2\':\'v2\'} == {\'k2\':\'v2\',\'k1\':\'v1\'} order should not matter for objects', async () => {
            const expression = "{'k1':'v1','k2':'v2'} == {'k2':'v2','k1':'v1'}";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression {\'k1\':\'v1\',\'k2\': {\'k1\':\'v1\',\'k2\':\'v2\'}} == {\'k2\': {\'k1\':\'v1\',\'k2\':\'v2\'},\'k1\':\'v1\'} order should not matter for objects', async () => {
            const expression = "{'k1':'v1','k2': {'k1':'v1','k2':'v2'}} == {'k2': {'k1':'v1','k2':'v2'},'k1':'v1'}";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression type(1) == type(4)', async () => {
            const expression = "type(1) == type(4)";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression type(type(1)) == type("string")', async () => {
            const expression = 'type(type(1)) == type("string")';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression size([1, 5]) == size([1, 2])', async () => {
            const expression = 'size([1, 5]) == size([1, 2])';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression size({"bla": 2, "blo": 4}) == size([1, 2])', async () => {
            const expression = 'size({"bla": 2, "blo": 4}) == size([1, 2])';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 2 + 2 + 6 == 10', async () => {
            const expression = '2 + 2 + 6 == 10';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 2 + 2 + 6 +2 == 10 + 2', async () => {
            const expression = '2 + 2 + 6 +2 == 10 + 2';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 2 in [2]', async () => {
            const expression = '2 in [2]';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Equals.false statements', async () => {
        it('Should return false when expression false == true', async () => {
            const expression = "false == true";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression true == false', async () => {
            const expression = "true == false";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 1 == 2', async () => {
            const expression = "1 == 2";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression banana == ba2ana', async () => {
            const expression = '"banana" == "ba2ana"';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression banana == ba3ana', async () => {
            const expression = '"banana" == "ba3ana"';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 1nana == r"12nana"', async () => {
            const expression = '"1anana" == r"12nana"';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 1 == 1.2', async () => {
            const expression = "1 == 1.2";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 1u == 1.2', async () => {
            const expression = "1u == 1.2";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 2 == 1u', async () => {
            const expression = "2 == 1u";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 2.0 == 1e+0', async () => {
            const expression = "2.0 == 1e+0";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 2 == 1.0e+0', async () => {
            const expression = "2 == 1.0e+0";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 2 == 3.0e+0', async () => {
            const expression = "2 == 3.0e+0";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression [13] == [1]', async () => {
            const expression = "[13] == [1]";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression [1.0, 2.1] == [1u, 2]', async () => {
            const expression = "[1.0, 2.1] == [1u, 2]";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression [1, 2, 3] == [1, 3, 2] respect the order', async () => {
            const expression = "[1, 2, 3] == [1, 3, 2]";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression [\'one\'] == [2, 3]', async () => {
            const expression = "['one'] == [2, 3]";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression [\'case\'] == [\'cAse\']', async () => {
            const expression = "['case'] == ['cAse']";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression [1, \'dos\', 3] == [1, 2, 4]', async () => {
            const expression = "[1, 'dos', 3] == [1, 2, 4]";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression [] == [1]', async () => {
            const expression = "[] == [1]";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression {\'k\':\'v\'} == {\'k\':\'v1\'}', async () => {
            const expression = "{'k':'v'} == {'k':'v1'}";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression {\'k\':\'v\',\'k1\':\'v1\'} == {\'k\':\'v\'}', async () => {
            const expression = "{'k':'v','k1':'v1'} == {'k':'v'}";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression type(1) == "string"', async () => {
            const expression = 'type(1) == "string"';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression size([1]) == size([1, 2])', async () => {
            const expression = 'size([1]) == size([1, 2])';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 2 in []', async () => {
            const expression = '2 in []';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Not Equals.true statements', async () => {
        it('Should return true when expression false != true', async () => {
            const expression = "false != true";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 != 2', async () => {
            const expression = "1 != 2";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression banana != banaana', async () => {
            const expression = '"banana" != "banaana"';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 != 1.2', async () => {
            const expression = "1 != 1.2";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Not Equals.false statements', async () => {
        it('Should return false when expression false != false', async () => {
            const expression = "false != false";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 1 != 1', async () => {
            const expression = "1 != 1";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression banana != banana', async () => {
            const expression = '"banana" != "banana"';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 1 != 1.0', async () => {
            const expression = "1 != 1.0";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Greater then or equal.true statements', async () => {
        it('Should return true when expression 1 >= 1', async () => {
            const expression = "1 >= 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1.2 >= 1', async () => {
            const expression = "1.2 >= 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 12 >= 1', async () => {
            const expression = "12 >= 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 12 >= 1.3', async () => {
            const expression = "12 >= 1.3";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression size([12, 3]) >= size([1200])', async () => {
            const expression = "size([12, 3]) >= size([1200])";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Greater then or equal.false statements', async () => {
        it('Should return false when expression 1 >= 12', async () => {
            const expression = "1 >= 12";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 1.2 >= 12', async () => {
            const expression = "1.2 >= 12";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 12 >= 12.1', async () => {
            const expression = "12 >= 12.1";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 12 >= 13', async () => {
            const expression = "12 >= 13";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Smaller then or equal.true statements', async () => {
        it('Should return true when expression 1 <= 1', async () => {
            const expression = "1 <= 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 <= 1.2', async () => {
            const expression = "1 <= 1.2";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 <= 12', async () => {
            const expression = "1 <= 12";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1.3 <= 12', async () => {
            const expression = "1.3 <= 12";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Smaller then or equal.false statements', async () => {
        it('Should return false when expression 12 <= 1', async () => {
            const expression = "12 <= 1";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 12 <= 1.2', async () => {
            const expression = "12 <= 1.2";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 12.1 <= 12', async () => {
            const expression = "12.1 <= 12";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 13 <= 12', async () => {
            const expression = "13 <= 12";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Greater then.true statements', async () => {
        it('Should return true when expression 1 > 0.1', async () => {
            const expression = "1 > 0.1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1.2 > 1', async () => {
            const expression = "1.2 > 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 12 > 1.3', async () => {
            const expression = "12 > 1.3";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Greater then.false statements', async () => {
        it('Should return false when expression 12 > 12', async () => {
            const expression = "12 > 12";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 1.2 > 12', async () => {
            const expression = "1.2 > 12";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 12 >= 12.1', async () => {
            const expression = "12 > 12.1";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Smaller then.true statements', async () => {
        it('Should return true when expression 1 < 1.1', async () => {
            const expression = "1 < 1.1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 1 < 2', async () => {
            const expression = "1 < 2";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 2.1 < 3', async () => {
            const expression = "2.1 < 3";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Smaller then.false statements', async () => {
        it('Should return false when expression 12 < 12', async () => {
            const expression = "12 < 12";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 12 < 1.2', async () => {
            const expression = "12 < 1.2";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression 12.1 < 12', async () => {
            const expression = "12.1 < 12";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });
});
