import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Conversions', async () => {
    let parser: any;
    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('bytes', async () => {
        it('Should return bytes_value empty when expression bytes(\'\')', async () => {
            const expression = "bytes('')";
            const expected = {bytes_value: ""};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return bytes_value abc when expression bytes(\'abc\')', async () => {
            const expression = "bytes('abc')";
            const expected = {bytes_value: "abc"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('double', async () => {
        it('Should return double_value empty when expression double(0)', async () => {
            const expression = "double(0)";
            const expected = {double_value: 0.0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return double_value 1 when expression double(1)', async () => {
            const expression = "double(1)";
            const expected = {double_value: 1.0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return double_value 1 when expression double(\'1\')', async () => {
            const expression = "double('1')";
            const expected = {double_value: 1.0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('int', async () => {
        it('Should return int64_value 42 when expression int(42u)', async () => {
            const expression = "int(42u)";
            const expected = {int64_value: 42};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return int64_value 0 when expression int(0u)', async () => {
            const expression = "int(0u)";
            const expected = {int64_value: 0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return int64_value 1 when expression int(1.9)', async () => {
            const expression = "int(1.9)";
            const expected = {int64_value: 1};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return int64_value 42 when expression int("42")', async () => {
            const expression = "int('42')";
            const expected = {int64_value: 42};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('uint', async () => {
        it('Should return uint64_value 42 when expression uint(42u)', async () => {
            const expression = "uint(42u)";
            const expected = {uint64_value: 42};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return uint64_value 0 when expression uint(0)', async () => {
            const expression = "uint(0)";
            const expected = {uint64_value: 0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return uint64_value 1 when expression uint(1.9)', async () => {
            const expression = "uint(1.9)";
            const expected = {uint64_value: 1};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return uint64_value 42 when expression int("42")', async () => {
            const expression = "uint('42')";
            const expected = {uint64_value: 42};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });
})