import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Conversions', async () => {
    let parser: any;
    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('map', async () => {
        it('Should return bool_value true when expression {\'0\':1,\'2\':2,\'5\':true}[5]', async () => {
            const expression = "{'0':1,'2':2,'5':true}['5']";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return uint64_value 100 when expression {\'name\':100u}[\'name\']', async () => {
            const expression = "{'name':100u}['name']";
            const expected = {uint64_value: 100};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('array', async () => {
        it('Should return int64_value 6 when expression [0,4,6][2]', async () => {
            const expression = "[0,4,6][2]";
            const expected = {int64_value: 6};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return double_value 1.3 when expression [0,1.3,6][1]', async () => {
            const expression = "[0,1.3,6][1]";
            const expected = {double_value: 1.3};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return double_value 1.3 when expression [0,1.3, 6][1]', async () => {
            const expression = "[0,1.3, 6][1]";
            const expected = {double_value: 1.3};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });
});