import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Math', async () => {
    let parser: any;
    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('Plus', async () => {
        it('Should return 4 int64_value when expression 2 + 2', async () => {
            const expression = "2 + 2";
            const expected = {int64_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 double_value when expression 2.0 + 2.0', async () => {
            const expression = "2.0 + 2.0";
            const expected = {double_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 6 int64_value when expression 2 + 2 + 2', async () => {
            const expression = "2 + 2 + 2";
            const expected = {int64_value: 6};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 65 int64_value when expression 22 + 22 + 10 + 5 + 6', async () => {
            const expression = "22 + 22 + 10 + 5 + 6";
            const expected = {int64_value: 65};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 6.6 double_value when expression 2.2 + 2.2 + 2.2', async () => {
            const expression = "2.2 + 2.2 + 2.2";
            const expected = {double_value: 6.6};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Minus', async () => {
        it('Should return 4 int64_value when expression 6 - 2', async () => {
            const expression = "6 - 2";
            const expected = {int64_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 double_value when expression 6.0 - 2.0', async () => {
            const expression = "6.0 - 2.0";
            const expected = {double_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 int64_value when expression 8 - 2 - 2', async () => {
            const expression = "8 - 2 - 2";
            const expected = {int64_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 62 int64_value when expression 120 - 30 - 5 - 16 - 7', async () => {
            const expression = "120 - 30 - 5 - 16 - 7";
            const expected = {int64_value: 62};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 double_value when expression 8.4 - 2.2 - 2.2', async () => {
            const expression = "8.4 - 2.2 - 2.2";
            const expected = {double_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Multiply', async () => {
        it('Should return 12 int64_value when expression 6 * 2', async () => {
            const expression = "6 * 2";
            const expected = {int64_value: 12};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 12 double_value when expression 6.0 * 2.0', async () => {
            const expression = "6.0 * 2.0";
            const expected = {double_value: 12};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 32 int64_value when expression 8 * 2 * 2', async () => {
            const expression = "8 * 2 * 2";
            const expected = {int64_value: 32};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 2016000 int64_value when expression 120 * 30 * 5 * 16 * 7', async () => {
            const expression = "120 * 30 * 5 * 16 * 7";
            const expected = {int64_value: 2016000};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 16.698 double_value when expression 2.2 * 2.3 * 3.3', async () => {
            const expression = "2.2 * 2.3 * 3.3";
            const expected = {double_value: 16.698};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Divide', async () => {
        it('Should return 3 int64_value when expression 6 / 2', async () => {
            const expression = "6 / 2";
            const expected = {int64_value: 3};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 3 double_value when expression 6.0 / 2.0', async () => {
            const expression = "6.0 / 2.0";
            const expected = {double_value: 3};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 2 int64_value when expression 8 / 2 / 2', async () => {
            const expression = "8 / 2 / 2";
            const expected = {int64_value: 2};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 0.0071428571428571435 int64_value when expression 120 / 30 / 5 / 16 / 7', async () => {
            const expression = "120 / 30 / 5 / 16 / 7";
            const expected = {int64_value: 0.0071428571428571435};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Combination', async () => {
        it('Should return 54 int64_value when expression 2 + 4 + 6 * 8', async () => {
            const expression = "2 + 4 + 6 * 8";
            const expected = {int64_value: 54};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 54 double_value when expression 2.0 + 4.0 + 6.0 * 8.0', async () => {
            const expression = "2.0 + 4.0 + 6.0 * 8.0";
            const expected = {double_value: 54};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 48 int64_value when expression 6 * 6 + 12', async () => {
            const expression = "6 * 6 + 12";
            const expected = {int64_value: 48};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 42 int64_value when expression 6 * 6 + 12 / 2', async () => {
            const expression = "6 * 6 + 12 / 2";
            const expected = {int64_value: 42};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 32 int64_value when expression 6 * 6 + 12 / 2 - 10', async () => {
            const expression = "6 * 6 + 12 / 2 - 10";
            const expected = {int64_value: 32};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Errors', async () => {
        it('when expression 2 + 4.3', async () => {
            const expression = "2 + 4.3";
            const expectedFound = "You are trying a mathematical expression with mismatching types";
            try {
                parser.parseString(expression);
            } catch(e: any) {
                expect(e.message).toBe(expectedFound);
            }
        });

        it('when expression 2 * 4.3', async () => {
            const expression = "2 * 4.3";
            const expectedFound = "You are trying a mathematical expression with mismatching types";
            try {
                parser.parseString(expression);
            } catch(e: any) {
                expect(e.message).toBe(expectedFound);
            }
        });

        it('when expression 2 - 4.3', async () => {
            const expression = "2 - 4.3";
            const expectedFound = "You are trying a mathematical expression with mismatching types";
            try {
                parser.parseString(expression);
            } catch(e: any) {
                expect(e.message).toBe(expectedFound);
            }
        });

        it('when expression 2 / 4.3', async () => {
            const expression = "2 / 4.3";
            const expectedFound = "You are trying a mathematical expression with mismatching types";
            try {
                parser.parseString(expression);
            } catch(e: any) {
                expect(e.message).toBe(expectedFound);
            }
        });

        it('when expression 2.2 * 4.3 + 3.0 * 2', async () => {
            const expression = "2.2 * 4.3 + 3.0 * 2";
            const expectedFound = "You are trying a mathematical expression with mismatching types";
            try {
                parser.parseString(expression);
            } catch(e: any) {
                expect(e.message).toBe(expectedFound);
            }
        });
    });
});
