import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Timestamp', async () => {
    let parser: any;

    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('Timestamp function', async () => {
        it('Should return seconds 1744236800 when expression timestamp("2023-28-09T21:13:20Z")', async () => {
            const expression = "timestamp(\"2023-28-09T21:13:20Z\")";
            const expected = {seconds: 1744226000};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return seconds 1744226000 when expression timestamp(\'2023-28-09T21:13:20Z\')', async () => {
            const expression = "timestamp('2023-28-09T21:13:20Z')";
            const expected = {seconds: 1744226000};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return seconds 1744226000 when expression timestamp(1744226000)', async () => {
            const expression = "timestamp(1744226000)";
            const expected = {seconds: 1744226000};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Timestamp equals', async () => {
        it('Should return true when expression timestamp(\'2009-02-13T23:31:30Z\') == timestamp(\'2009-02-13T23:31:30Z\')', async () => {
            const expression = "timestamp('2009-02-13T23:31:30Z') == timestamp('2009-02-13T23:31:30Z')";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression timestamp("2009-02-13T23:31:30Z") == timestamp(\'2009-02-13T23:31:30Z\')', async () => {
            const expression = "timestamp(\"2009-02-13T23:31:30Z\") == timestamp('2009-02-13T23:31:30Z')";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression timestamp("2009-02-13T23:31:30Z") == timestamp(1234564290)', async () => {
            const expression = "timestamp(\"2009-02-13T23:31:30Z\") == timestamp(1234564290)";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression timestamp("2009-02-13T23:31:30Z") != timestamp(1234564290)', async () => {
            const expression = "timestamp(\"2009-02-13T23:31:30Z\") != timestamp(1234564290)";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression timestamp(\'2009-02-13T23:31:30Z\') == timestamp(\'2009-02-13T23:31:29Z\')', async () => {
            const expression = "timestamp('2009-02-13T23:31:30Z') == timestamp('2009-02-13T23:31:29Z')";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression timestamp(\'2009-02-13T23:31:30Z\') != timestamp(\'2009-02-13T23:31:29Z\')', async () => {
            const expression = "timestamp('2009-02-13T23:31:30Z') != timestamp('2009-02-13T23:31:29Z')";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Timestamp comparison', async () => {
        it('Should return true when expression timestamp(\'2009-02-13T23:31:30Z\') <= timestamp(\'2009-02-13T23:31:30Z\')', async () => {
            const expression = "timestamp('2009-02-13T23:31:30Z') <= timestamp('2009-02-13T23:31:30Z')";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression timestamp("2009-02-13T23:31:30Z") >= timestamp(\'2009-02-13T23:31:30Z\')', async () => {
            const expression = "timestamp(\"2009-02-13T23:31:30Z\") >= timestamp('2009-02-13T23:31:30Z')";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression timestamp(\'2009-02-13T23:31:30Z\') < timestamp(\'2009-02-13T23:31:30Z\')', async () => {
            const expression = "timestamp('2009-02-13T23:31:30Z') < timestamp('2009-02-13T23:31:30Z')";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression timestamp(\'2009-02-13T23:31:30Z\') > timestamp(\'2009-02-13T23:31:30Z\')', async () => {
            const expression = "timestamp('2009-02-13T23:31:30Z') > timestamp('2009-02-13T23:31:30Z')";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression timestamp(\'2009-02-13T23:31:31Z\') > timestamp(\'2009-02-13T23:31:30Z\')', async () => {
            const expression = "timestamp('2009-02-13T23:31:31Z') > timestamp('2009-02-13T23:31:30Z')";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });
});
