import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Type', async () => {
    let parser: any;
    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('Class.check', async () => {
        it('Should return int_value when expression type(1)', async () => {
            const expression = "type(1)";
            const expected = {class_value: "int64_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return boolean when expression type(true)', async () => {
            const expression = "type(true)";
            const expected = {class_value: "bool_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return null_value when expression type(null)', async () => {
            const expression = "type(null)";
            const expected = {class_value: "null_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return double_value when expression type(1.2)', async () => {
            const expression = "type(1.2)";
            const expected = {class_value: "double_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return uint_value when expression type(12u)', async () => {
            const expression = "type(12u)";
            const expected = {class_value: "uint64_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return string_value when expression type("banana")', async () => {
            const expression = "type(\"banana\")";
            const expected = {class_value: "string_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return bytes_value when expression type(b"")', async () => {
            const expression = "type(b\"\")";
            const expected = {class_value: "bytes_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return list_value when expression type([])', async () => {
            const expression = "type([])";
            const expected = {class_value: "list_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return list_value when expression type([1, 4, 5])', async () => {
            const expression = "type([1, 4, 5])";
            const expected = {class_value: "list_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return map_value when expression type({})', async () => {
            const expression = "type({})";
            const expected = {class_value: "map_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return map_value when expression type({"bla": 1})', async () => {
            const expression = "type({\"bla\": 1})";
            const expected = {class_value: "map_value"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });
});
