{
  function getValueFromKeys(keys) {
    let result = options.values;
    keys.forEach(key => {
      if (result && result.hasOwnProperty(key)) {
        result = result[key];
      } else {
        result = undefined;
      }
    });
    return result;
  }

  function compareTypes(operator, left, right) {
   switch(operator) {
     case "==":
      return {bool_value: (left === right)}
     case "!=":
      return {bool_value: left !== right}
     case "<":
      return {bool_value: left < right}
     case ">":
      return {bool_value: left > right}
     case "<=":
      return {bool_value: left <= right}
     case ">=":
      return {bool_value: left >= right}
     case "&&":
      return {bool_value: left && right}
     case "||":
      return {bool_value: left || right}
   }
  }

  function setType(key, value) {
    if(key == "class_value") {
      return "string_value";
    } else {
      return key;
    }
  }

  function objectType(object) {
    if(Object.keys(object)[0] === "map_value") {
      return "map";
    } else if(Object.keys(object)[0] === "list_value") {
      return "list";
    }
  }

  function compareArrays(left, right) {
    if(left.list_value.values === undefined && right.list_value.values === undefined) {
      return true;
    } else if (left.list_value.values !== undefined && right.list_value.values === undefined || left.list_value.values === undefined && right.list_value.values !== undefined) {
      return false;
    }
    const leftArray = left.list_value.values.map(item => Object.values(item)[0]);
    const rightArray = right.list_value.values.map(item => Object.values(item)[0]);
    if(leftArray.length !== rightArray.length) {
      return false;
    } else {
       return leftArray.every((value, index) => value === rightArray[index]);
    }
  }

  function compareCompleteObject(left, right) {
    if(left.map_value.entries === undefined && right.map_value.entries === undefined) {
      return true;
    } else if (left.map_value.entries !== undefined && right.map_value.entries === undefined || left.map_value.entries === undefined && right.map_value.entries !== undefined) {
      return false;
    }
    const leftEntries = transformToComparableObject(left.map_value.entries);
    const rightEntries = transformToComparableObject(right.map_value.entries);
    if(leftEntries.length !== rightEntries.length) {
      return false;
    } else {
       return compareTranslatedObject(leftEntries, rightEntries);
    }
  }

  function compareTranslatedObject(left, right) {
    for (const key in left) {
     if(typeof(left[key]) == 'object' && typeof(right[key]) == 'object') {
       compareTranslatedObject(left[key], right[key])
     } else {
       if(left[key] != right[key]) {
         return false;
       }
     }
   }
   return true;
  }

  function transformToComparableObject(object) {
    const result = {};
    object.forEach(item => {
      const value = Object.values(item.value)[0];
      if(value.entries == undefined) {
        result[Object.values(item.key)[0]] = value;
      } else {
        result[Object.values(item.key)[0]] = transformToComparableObject(value.entries)
      }
    });
    return result;
  }

  function transformToObjectToGetWithKey(object) {
    const result = {};
    object.forEach(item => {
      const value = item.value;
      if(value.entries == undefined) {
        result[Object.values(item.key)[0]] = value;
      } else {
        result[Object.values(item.key)[0]] = transformToComparableObject(value.entries)
      }
    });
    return result;
  }

  function setTypeAndValue(value) {
     if(value.isLosslessNumber) {
       if (value.value.includes('.')) {
         return {double_value: value.valueOf()};
       } else {
         return {int64_value: value.valueOf()};
       }
    }
    if(typeof(value) === 'string') {
      return {string_value: value};
    } else if(typeof(value) === 'number') {
      if (value.toString().includes('.')) {
        return {double_value: value};
      } else {
        return {int64_value: value};
      }
    } else if(typeof(value) === 'boolean') {
      return {bool_value: value};
    } else if(typeof(value) === 'object' && value.constructor === Object) {
      let entries = [];
      for (const [k, v] of Object.entries(value)) {
        let entry = {key: setTypeAndValue(k), value: setTypeAndValue(v)}
        entries.push(entry);
      }
      return {map_value: {entries: entries}}
    } else if(typeof(value) === 'object' && value.constructor === Array) {
      let entries = [];
      for (const item of value) {
        let entry = setTypeAndValue(item)
        entries.push(entry);
      }
      return {list_value: {values: entries}}
    }
  }

  function possibleExpressionType(key) {
    return key && (key === 'double_value' || key === 'int64_value');
  }

  function checkMatchingType(firstElement, secondElement) {
    return Object.keys(firstElement)[0] === Object.keys(secondElement)[0];
  }

  function setTypeAndValueWithKnownType(type, value) {
    if(type === 'string_value') {
      return {string_value: value};
    } else if(type === 'int64_value') {
      return {int64_value: value};
    } else if(type === 'double_value') {
      return {double_value: Number(value.toFixed(3))};
    }
  }
}

Start = _ final:ConditionalBooleanStatement _ {
  return final;
 }

ConditionalBooleanStatement
  = condition:OrCondition _ "?" _ body_true:ConditionalBooleanStatement _ ":" _ body_false:ConditionalBooleanStatement {
   if(Object.values(condition)[0]) {
    return body_true;
   } else {
   	return body_false;
   }
  }
  / OrCondition
  / CompareEqualsOptions
  / CompareNormalTypes
  / Factor
  / Expression

OrCondition = head:AndCondition tail:(_ '||' _ AndCondition)* {
  return tail.reduce(function(result, element) {
    return compareTypes('||', Object.values(result)[0], Object.values(element[3])[0]);
  }, head);
} / AndCondition

AndCondition = head:CompareEqualsOptions tail:(_ '&&' _ CompareEqualsOptions)* {
  return tail.reduce(function(result, element) {
    return compareTypes('&&', Object.values(result)[0], Object.values(element[3])[0]);
  }, head);
} / CompareEqualsOptions

CompareEqualsOptions
  = CompareArrayEquals
  / CompareObjectEquals
  / HasMacro
  / NotLogic
  / CompareNormalTypes
  / InCheck
  / Expression
  / CustomObject
  / true
  / false

CompareNormalTypes = left:CompareNormal _ compare:Operator _ right:CompareNormal {
    return compareTypes(compare, Object.values(left)[0], Object.values(right)[0])
  }

Expression
  = head:Term tail:(_ (plus / minus) _ Term)* {
      let result = tail.reduce(function(result, element) {
        if(!checkMatchingType(head, tail[0][3])) {
          throw new Error("You are trying a mathematical expression with mismatching types");
        }
        let number;
        if(typeof result == 'number'){
          number = result;
        } else if (possibleExpressionType(Object.keys(result)[0])) {
          number = Object.values(result)[0];
        } else {
          throw new Error("You are trying a mathematical expression with the wrong type");
        }
        if(!possibleExpressionType(Object.keys(element[3])[0])){
          throw new Error("You are trying a mathematical expression with the wrong type");
        }
        if (element[1] === "+") {
          return number + Object.values(element[3])[0];
        }
        if (element[1] === "-") {
          return number - Object.values(element[3])[0];
        }
      }, head);
      if(typeof result == 'number'){
        let type = Object.keys(head)[0];
        return setTypeAndValueWithKnownType(type, result)
      } else {
        return result;
      }
    }

Term
  = head:Factor tail:(_ (multiply / divide) _ Factor)* {
      return tail.reduce(function(result, element) {
        let type = Object.keys(head)[0];
        if(!checkMatchingType(head, tail[0][3])) {
          throw new Error("You are trying a mathematical expression with mismatching types");
        }
        if (element[1] === "*") {
          let endResult = Object.values(result)[0] * Object.values(element[3])[0];
          return setTypeAndValueWithKnownType(type, endResult);
        }
        if (element[1] === "/") {
          let endResult = Object.values(result)[0] / Object.values(element[3])[0];
          return setTypeAndValueWithKnownType(type, endResult);
        }
      }, head);
    }

Factor
  = "(" _ expr:Expression _ ")" { return expr; }
  / CustomObject
  / SizeMacro
  / TypeCheck
  / DynType
  / Timestamp
  / AllTypes


TypeCheck
  = "type(" type:AllTypes ")" {
    let key = Object.keys(type)[0];
    let value = Object.values(type)[0];
    return {class_value: setType(key, value)}
  }

HasMacro
  = "has(" _ object:Objects ("." / "," _) key:[a-z0-9]+ _ ")" {
   let keyFull = key.join("");
   let present = false;
   if(objectType(object) == "map") {
     object["map_value"]["entries"].forEach(function (item) {
       if(item["key"]["string_value"] === keyFull) {
         present = true;
       }
     });
   } else {
     object["list_value"]["values"].forEach(function (item) {
     if(Object.values(item)[0].toString() === keyFull) {
       present = true;
     }
   })
  }
  return {bool_value: present}
  } / "has(" _ custom:CustomObject "," _ key:[a-z0-9]+ _ ")" {
    let keyFull = key.join("")
    let present = false;
    let object = Object.values(custom)[0];
    if(object.entries) {
      object.entries.forEach(function (item) {
        if(item["key"]["string_value"] === keyFull) {
         present = true;
       }
      })
    } else if(object.values) {
      object.values.forEach(function (item) {
        if(Object.values(item)[0].toString() === keyFull) {
          present = true;
        }
      })
    }
    return {bool_value: present}
  }

SizeMacro
  = "size(" _ object:Objects _ ")" {
    let length;
    if(objectType(object) == "map") {
      length = (object["map_value"]["entries"]) ? object["map_value"]["entries"].length : 0
      return {int64_value: length}
    } else {
      length = (object["list_value"]["values"]) ? object["list_value"]["values"].length : 0
      return {int64_value: length}
    }
  } / "size(" _ string:(string_literal / raw_string / raw_string_single_quotes) _ ")" {
    return {int64_value: Object.values(string)[0].length}
  } / "size(" _ custom:CustomObject _ ")" {
    let object = Object.values(custom)[0];
    if(object.entries) {
      return {int64_value: object.entries.length}
    } else if(object.values) {
      return {int64_value: object.values.length}
    } else {
      return {int64_value: Object.values(custom)[0].length}
    }
  }

BytesConversion
 = "bytes(" _ string:string_literal _ ")" {
   return { bytes_value: Object.values(string)[0] }
 }

DoublesConversion
 = "double(" _ variable:(unsigned_integer / integer / string_literal) _ ")" {
   return { double_value: parseFloat(Object.values(variable)[0]) }
 }

IntConversion
 = "int(" _ variable:(double / unsigned_integer / integer / string_literal) _ ")" {
   return { int64_value: parseInt(Object.values(variable)[0]) }
 }

UIntConversion
 = "uint(" _ variable:(double / unsigned_integer / integer / string_literal) _ ")" {
   return { uint64_value: parseInt(Object.values(variable)[0]) }
 }

DynType
 = "dyn(" _ type:BasicTypes _ ")" {
   return {dyn_value: Object.values(type)[0]}
 }

Timestamp
  = "timestamp('" year:Year  "-" month:Month "-" day:Day "T" hour:Hour ":" minute:Minute  ":" second:Second "Z')" {
      return {seconds: (new Date(year, month - 1, day, hour, minute, second) / 1000)};
    } / 'timestamp("' year:Year "-" month:Month "-" day:Day "T" hour:Hour ":" minute:Minute  ":" second:Second 'Z")' {
      return {seconds: (new Date(year, month - 1, day, hour, minute, second) / 1000)};
    } / 'timestamp(' seconds:integer ')' {
      return {seconds: Object.values(seconds)[0]};
    }

Year
  = DIGIT DIGIT DIGIT DIGIT {
    return parseInt(text(), 10);
  }

Month
  = DIGIT DIGIT {
    return parseInt(text(), 10);
  }

Day
  = DIGIT DIGIT {
    return parseInt(text(), 10);
  }

Hour
  = DIGIT DIGIT {
    return parseInt(text(), 10);
  }

Minute
  = DIGIT DIGIT {
    return parseInt(text(), 10);
  }

Second
  = DIGIT DIGIT {
    return parseInt(text(), 10);
  }

NotLogic = "!" statement:(CompareEqualsOptions / "(" CompareEqualsOptions ")") {
  let boolean;
  if(statement.length != undefined) {
    boolean = Object.values(statement[1])[0];
  } else {
    boolean = Object.values(statement)[0];
  }
  return {bool_value: !boolean}
}

CompareArrayEquals = left:(EmptyArray/Array) _ compare:Operator _ right:(EmptyArray/Array) {
  let boolean;
  if(compare === "=="){
    boolean = compareArrays(left, right);
  } else {
    boolean = !compareArrays(left, right);
  }
  return {bool_value: boolean};
}

CompareObjectEquals = left:(EmptyMap/Map) _ compare:Operator _ right:(EmptyMap/Map) {
  let boolean;
  if(compare === "=="){
    boolean = compareCompleteObject(left, right);
  } else {
    boolean = !compareCompleteObject(left, right);
  }
  return {bool_value: boolean};
}

Operator = op:("==" / ">=" / "<="  / "!=" / ">" / "<")

InCheck = check:AllTypes _ "in" _ items:Array {
    return { bool_value:items["list_value"]["values"].map(i => Object.values(i)[0]).includes( Object.values(check)[0]) };
  }
  / AllTypes _ "in" _ EmptyArray {
    return { bool_value: false };
  }
  / check:AllTypes _ "in" _ items:CustomObject {
    return { bool_value:items["list_value"]["values"].map(i => Object.values(i)[0]).includes( Object.values(check)[0]) };
  }

MapField
  = map:Map "[" _ key:BasicTypes _ "]" {
    return transformToObjectToGetWithKey(map.map_value.entries)[Object.values(key)[0]];
  }

ArrayItem
  = array:Array "[" _ position:(unsigned_integer / integer) _ "]" {
    return array.list_value.values[Object.values(position)[0]];
  }

Objects
 = EmptyMap
 / EmptyArray
 / Map
 / Array

CustomObject
 = first:[_a-zA-Z]+  keys:("." [_a-zA-Z]+)+ {
   let array = [];
   const firstKey = first.join("");
   array.push(firstKey);
   keys.forEach(e => array.push(e[1].join("")))
   return setTypeAndValue(getValueFromKeys(array))
 }

EmptyMap = "{}" {return {map_value: {}}}

Map
  = "{" _ keyValues:(p:Pair _ ","? _ { return p})* _ "}" {
      return {map_value: {entries: keyValues.reduce(function(obj, element) {
        return obj
       }, keyValues)
      }
    }
  }

Pair
  = key:string_literal _ ":" _ value:AllTypes {
    return {key: key, value: value}
  }

EmptyArray = "[]" {return {list_value: {}};}

Array = "[" item:AllTypes items:("," _ AllTypes)* "]" {
  let array = [];
  array.push(item);
  items.forEach(e => {
    array.push(e[2]);
  })
  return {list_value: {values: array}};
}

CompareNormal
  = "(" _ expr:Expression _ ")" { return expr; }
  / Expression
  / "(" _ term:Term _ ")" { return term; }
  / Term
  / MapField
  / ArrayItem
  / CustomObject
  / DynType
  / Timestamp
  / BasicTypes
  / TypeCheck
  / SizeMacro
  / BytesConversion
  / DoublesConversion
  / IntConversion
  / UIntConversion

AllTypes
  = MapField
  / ArrayItem
  / Objects
  / CustomObject
  / DynType
  / Timestamp
  / BasicTypes
  / TypeCheck
  / SizeMacro
  / BytesConversion
  / DoublesConversion
  / IntConversion
  / UIntConversion

BasicTypes
 = false
  / true
  / null
  / unsigned_positive_hex
  / unsigned_negative_hex
  / positive_hex
  / negative_hex
  / double
  / unsigned_integer
  / integer
  / raw_string
  / raw_string_single_quotes
  / bytes_string
  / bytes_string_single_quotes
  / string_literal

false = "false" { return { bool_value: false }; }
null  = "null"  { return { null_value: null };  }
true  = "true"  { return { bool_value: true };  }

positive_hex
  = "0x" hex:HEXDIG+ {
    return { int64_value: parseInt(hex.join(""), 16)}
  }

negative_hex
  = "-0x" hex:HEXDIG+ {
    return { int64_value: 0 - parseInt(hex.join(""), 16)}
  }

unsigned_positive_hex
 = "0x" hex:HEXDIG+ ("u" / "U") {
    return { uint64_value: parseInt(hex.join(""), 16)}
  }

unsigned_negative_hex
  = "-0x" hex:HEXDIG+ ("u" / "U") {
    return { uint64_value: 0 - parseInt(hex.join(""), 16)}
  }

double
  = minus? (int / '' ) frac exp? { return { double_value: parseFloat(text())}; }

integer
  = minus? int exp? { return { int64_value: parseFloat(text()) }; }

unsigned_integer
  = int exp? ('u' / 'U') { return { uint64_value: parseFloat(text()) }; }

raw_string
  = 'r"' string:DoubleStringCharacter* '"' { return {string_value: string.join("") } }

raw_string_single_quotes
  = "r'" string:SingleStringCharacter* "'" { return {string_value: string.join("") } }

bytes_string
  = 'b"' chars:DoubleStringCharacter* '"' {
  const bytesArr = []
  for (let i = 0; i < chars.length; i++) {
  	bytesArr.push(chars[i].charCodeAt(0))
  }
  let endString = "";
  if(bytesArr.length > 0) {
    endString = "\\" + bytesArr.join('\\')
  }
  return {bytes_value: endString}
  }

bytes_string_single_quotes
  = "b'" chars:SingleStringCharacter* "'" {
  const bytesArr = []
  for (let i = 0; i < chars.length; i++) {
  	bytesArr.push(chars[i].charCodeAt(0))
  }
  let endString = "";
  if(bytesArr.length > 0) {
    endString = "\\" + bytesArr.join('\\')
  }
  return {bytes_value: endString}
  }

string_literal
  = '"' chars:DoubleStringCharacter* '"' {
    let str = chars.join('');
    return { string_value: chars.join('') };
  }
  / "'" chars:SingleStringCharacter* "'" { return { string_value: chars.join('') }; }

DoubleStringCharacter
  = !('"' / "\\") char:. { return char; }
  / "\\" sequence:EscapeSequence { return sequence; }

SingleStringCharacter
  = !("'" / "\\") char:. { return char; }
  / "\\" sequence:EscapeSequence { return sequence; }

EscapeSequence
  = "'"
  / '"'
  / "\\"
  / "b"  { return "\b";   }
  / "f"  { return "\f";   }
  / "n"  { return "\n";   }
  / "r"  { return "\r";   }
  / "t"  { return "\t";   }
  / "v"  { return "\x0B"; }

decimal_point
  = "."

digit1_9
  = [1-9]

e
  = [eE]

exp
  = e (minus / plus)? DIGIT+

frac
  = decimal_point DIGIT+

int
  = zero / (digit1_9 DIGIT*)

divide
  = "/"

multiply
  = "*"

minus
  = "-"

underscore
  = "_"

plus
  = "+"

zero
  = "0"

DIGIT  = [0-9]
HEXDIG = [0-9a-f]i

_ "whitespace"
  = [ \t\n\r]*
